<?php

namespace Goldenscarab\Modulus\Service\Template\Compose;

use Illuminate\Support\Arr;
use Illuminate\Support\Str;
use \Illuminate\Support\HtmlString;
use Goldenscarab\Modulus\Helpers\Dynamik;
use Goldenscarab\Modulus\Helpers\Bootstrap4;


class Template
{
    /**
     * Prefixe des routes
     *
     * @var string
     */
    protected $route_prefix;

    /**
     * Prefixe des permission
     *
     * @var string
     */
    protected $can_prefix;

    /**
     * Source des données
     *
     * @var Collection
     */
    protected $source;

    /**
     * Id de la table
     *
     * @var string
     */
    protected $table_id;

    /**
     * Classe de la table
     *
     * @var string
     */
    protected $table_class;

    /**
     * Menu de la liste
     *
     * @var array
     */
    protected $_menu;

    /**
     * Configuration des lignes
     *
     * @var array
     */
    protected $_rows;

    /**
     * Configuration des colonnes
     *
     * @var array
     */
    protected $_columns;



    public function __construct($attrs = [])
    {
        $this->setAttrs($attrs);
    }


    /**
     * Replacement des tag de valeur par défaut par une action associée
     *
     * @param array $datas
     * @return array
     */
    protected function replaceDefaultsAction(array $datas)
    {
        $datas_default = [];

        $models_action = array(
            '[SHOW]' => [
                'label'      => '<i class="fa fa-eye mr-2"></i>' . __('Voir'),
                'permission' => $this->can_prefix . '-read',
                'href'       => ['url'    => [
                    'route'     => $this->route_prefix . '.show',
                    'args'      => [':id', ['callback' => ['function' => 'current_query_to_string()']]]
                ]],
            ],
            '[READ]' => [
                'label'      => '<i class="fa fa-eye mr-2"></i>' . __('Consulter'),
                'permission' => $this->can_prefix . '-read',
                'href'       => ['url'    => [
                    'route'     => $this->route_prefix . '.read',
                    'args'      => [':id', ['callback' => ['function' => 'current_query_to_string()']]]
                ]],
            ],
            '[CREATE]' => [
                'label'      => '<i class="fa fa-plus mr-2"></i>' . __('Ajouter'),
                'permission' => $this->can_prefix . '-create',
                'href'       => ['url'    => [
                    'route'     => $this->route_prefix . '.create',
                    'args'      => [':id', ['callback' => ['function' => 'current_query_to_string()']]]
                ]],
            ],
            '[UPDATE]' => [
                'label'      => '<i class="fa fa-pencil-square-o mr-2"></i>' . __('Modifier'),
                'permission' => $this->can_prefix . '-update',
                'href'       => ['url'    => [
                    'route'     => $this->route_prefix . '.update',
                    'args'      => [':id', ['callback' => ['function' => 'current_query_to_string()']]]
                ]],
            ],
            '[DUPLICATE]' => [
                'label'      => '<i class="fa fa-clone mr-2"></i>' . __('Dupliquer'),
                'permission' => $this->can_prefix . '-create',
                'href'       => ['url'    => [
                    'route'     => $this->route_prefix . '.duplicate',
                    'args'      => [':id', ['callback' => ['function' => 'current_query_to_string()']]]
                ]],
            ],
            '[DELETE]' => [
                'label'      => '<i class="fa fa-trash mr-2"></i>' . __('Supprimer'),
                'permission' => $this->can_prefix . '-delete',
                'href'       => '#',
                'class'      => 'confirm text-danger',
                'data-href'  => ['url'    => [
                    'route'     => $this->route_prefix . '.delete',
                    'args'      => [':id', ['callback' => ['function' => 'current_query_to_string()']]]
                ]],
            ],
            '[EXPORT]' => [
                'label'      => '<i class="fa fa-file-excel-o mr-2"></i>' . __('Exporter'),
                'permission' => $this->can_prefix . '-read',
                'href'       => ['url'    => [
                    'route'     => $this->route_prefix . '.export',
                    'args'      => [['callback' => ['function' => 'current_query_to_string()']]]
                ]],
            ],
            '[TRUNCATE]' => [
                'label'      => '<i class="fa fa-times mr-2"></i>' . __('Tout supprimer'),
                'permission' => $this->can_prefix . '-delete',
                'href'       => '#',
                'class'      => 'confirm',
                'data-href'  => ['url'    => [
                    'route'     => $this->route_prefix . '.truncate',
                    'args'      => [['callback' => ['function' => 'current_query_to_string()']]]
                ]],
            ],
            '[RESET-POSITION]' => [
                'label'      => '<i class="fa fa-list-ol mr-2"></i>' . __('Reset position'),
                'permission' => $this->can_prefix . '-update',
                'href'       => '#',
                'class'      => 'confirm',
                'data-href'  => ['url'    => [
                    'route'     => $this->route_prefix . '.reset-position',
                    'args'      => [['callback' => ['function' => 'current_query_to_string()']]]
                ]],
            ],
            '[SEPARATOR]' => '<div class="dropdown-divider"></div>'
        );

        // Recherche des tags de modèles à remplacer
        foreach ($datas as $key => $data) {

            if (is_array($data)) {

                $datas_default[$key] = $this->replaceDefaultsAction($data);
            } else {
                if (array_key_exists($data, $models_action)) {
                    //throw new \InvalidArgumentException("Modèle d'action non reconnu : " . $data);
                }

                $datas_default[$key] = data_get($models_action, $data, $data);
            }

        }

        return $datas_default;
    }

    /**
     * Retire les élements d'un tableau lorsqu'une permission est spécifié et que l'utilisateur ne dispose pas des droits
     *
     * @param array $datas
     * @return array
     */
    protected function removeWithoutPermission(array $datas)
    {
        $datas_permission = [];
        foreach ($datas as $key => $data) {

            if ($key === 'permission' && !auth()->user()->can($data)) {
                return null;
            }

            if (is_array($data)) {
                $datas_permission[$key] = $this->removeWithoutPermission($data);
            } else {
                $datas_permission[$key] = $data;
            }

        }

        // Filtre des éléments vides
        $datas_permission = array_filter($datas_permission, function($item) {
            return !is_null($item) && $item !== '';
        });

        return $datas_permission;
    }

    /**
     * Retourne le rendu d'attributs avec leur valeur
     * @param  array  $attrs Les attributs avec la colonne de l'object conteant la valeur
     * @param  Object $item  Object contenant la valeur de l'attribut
     * @return string        Le rendu des attributs avec leur valeur
     */
    protected function renderAttributes($attrs = [], $source = null): string
    {
        $render = '';

        if (is_iterable($attrs)) {
            $template = ' %s="%s"';
            foreach ($attrs as $name => $data) {
                $value   = $this->getValue($data, $source);
                $render .= vsprintf($template, [$name, $value]);
            }
        }

        return $render;
    }

    /**
     * Récupération d'une valeur en fonction de sa composition
     * @param  Mixed $value             array pour une valeur dynamique ou string|int|float|bool pour une valeur statique
     * @param  Object $datas            Objet dans lequel aller chercher la valeur si dynamique
     * @param  boolean $strip           Supprimer les balises HTML dela valeur résultante
     * @return Mixed                    La valeur
     */
    protected function getValue($value, $datas = null)
    {
        $dynamik = new Dynamik($datas);
        return $dynamik->getValue($value);
    }

    /**
     * Retourne le rendu d'une chaine de caractère en HTML
     * @param  String $string La chains de caractères à traiter
     * @return String         Le rendu HTML
     */
    public function renderHTML($string): string
    {
        return new HtmlString($string);
    }

    /**
     * Retourne le rendu avec indentation d'un texte
     * @param  string $text  Le texte à traiter
     * @param  Integer $nb   Le nombre d'indentation à appliquer
     * @return string        Le rendu avec indentation
     */
    protected function renderIndent($text, $nb): string
    {
        $indent = '    ';
        $render = '';

        $real_indent = '';

        for ($i = 1; $i <= $nb; $i++) {
            $real_indent .= $indent;
        }

        $lines = explode(PHP_EOL, $text);

        foreach ($lines as $line) {

            if (!empty($line)) {
                $render .= $real_indent . $line . PHP_EOL;
            }
        }

        return $render;
    }

    /**
     * Mise à jour des attributs depuis le tableau passé en paramètre
     * @param Array $attrs Liste des attributs à traiter
     * @return void
     */
    protected function setAttrs($attrs)
    {
        $this->resetAttrs();
        if (is_null($attrs)) {
            return ;
        }

        // Liste des attributs autorisés
        $attrs_available = array_map(function ($val) {
            return trim($val, '_');
        }, array_keys(get_object_vars($this)));

        // Traitement des attributs
        if (is_array($attrs)) {
            foreach ($attrs as $attr => $value) {
                if (in_array($attr, $attrs_available)) {
                    if (is_array($value)) {
                        $this->{'_' . $attr} = $value;
                    } else {
                        $this->{$attr} = $value;
                    }
                } else {
                    throw new \InvalidArgumentException("Le paramètre [" . $attr . "] n'est pas pris en charge");
                }
            }
        } else {
            throw new \InvalidArgumentException("Type de paramètre invalide. Attendu : Tableau");
        }

    }

    /**
     * Réinitialisation des attributs de constructions
     * @return void
     */
    protected function resetAttrs()
    {
        $attrs = array_keys(get_object_vars($this));

        foreach ($attrs as $attr) {

            if (preg_match('/^_.*/', $attr)) {
                $this->{$attr} = [];
            } else if (preg_match('/^is_.*/', $attr)) {
                $this->{$attr} = false;
            } else {
                $this->{$attr} = null;
            }
        }
    }
}
